#include "rectangles.h"
#include "editor.h"
#include "../common/sources/system/cgraphics2d.h"
#include <QDebug>

Rectangles   rectangles;

//---------------------------------------------------------------------------------------------
// draws a rectangle surrounding a tile for cursor and selection
void   Rectangles::drawTileRect(QImage* image, CVec2 mousePos, QColor color)
{
    CVec2   pos = mousePos / TILE_SIZE;
    CVec2   pos2 = pos * TILE_SIZE;

    CRect   r(CVec2(pos2), CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
    graph2D.rect(image, r, color, false);
}

//---------------------------------------------------------------------------------------------
// draws a rectangle surrounding a wall for cursor and selection
void   Rectangles::drawWallRect(QImage* image, CVec2 mousePos, QColor color)
{
    CVec2   pos = mousePos / TILE_SIZE;
    CVec2   pos2 = pos * TILE_SIZE;
    CRect   r;

    CVec2   localPos = mousePos - pos2;
    EWallSide   side = editor.getWallSide(localPos);

    switch (side)
    {
        case eWallSideUp:
            r = CRect(pos2, CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE / 4 - 1));
            break;

        case eWallSideDown:
            r = CRect(CVec2(pos2.x, pos2.y + (TILE_SIZE * 3) / 4),
                      CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
            break;

        case eWallSideLeft:
            r = CRect(pos2, CVec2(pos2.x + TILE_SIZE / 4 - 1, pos2.y + TILE_SIZE - 1));
            break;

        case eWallSideRight:
            r = CRect(CVec2(pos2.x + (TILE_SIZE * 3) / 4, pos2.y),
                      CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
            break;

        default:
            break;
    }

    graph2D.rect(image, r, color, false);
}

//---------------------------------------------------------------------------------------------
// draws a rectangle surrounding an object for cursor and selection
void   Rectangles::drawObjectRect(QImage* image, CVec2 mousePos, QColor color)
{
    CRect   r;

    CVec2   localPos = mousePos;
    localPos /= TILE_SIZE / 2;
    localPos *= TILE_SIZE / 2;

    r = CRect(localPos, CVec2(localPos.x + TILE_SIZE / 2 - 1, localPos.y + TILE_SIZE / 2 - 1));
    graph2D.rect(image, r, color, false);
}

//---------------------------------------------------------------------------------------------
void Rectangles::drawCursor(QImage* image, CVec2 mousePos, ETabIndex tabIndex)
{
    if (mousePos.x != -1)
    {
        if (tabIndex == eTabTiles)
            drawTileRect(image, mousePos, CURSOR_COLOR);
        else if (tabIndex == eTabWalls)
            drawWallRect(image, mousePos, CURSOR_COLOR);
        else if (tabIndex == eTabObjects)
            drawObjectRect(image, mousePos, CURSOR_COLOR);

        QString text = editor.getPositionString(mousePos, tabIndex);
        bridge.set_cursorInfo(text);
    }
    else
    {
        bridge.set_cursorInfo(QString());
    }
}

//---------------------------------------------------------------------------------------------
// draw the selection when we are in tile mode
void   Rectangles::drawTileSelection(QImage* image)
{
    CVec2   startPos = editor.mSelectStart / TILE_SIZE;
    CVec2   endPos   = editor.mSelectEnd   / TILE_SIZE;

    if (startPos == endPos)
    {
        drawTileRect(image, editor.mSelectStart, SELECTION_COLOR);
        bridge.set_tileSelScrollVis(true);
    }
    else
    {
        CRect   sel = editor.getSelectionRect();
        drawRectangleSelection(image, sel);
    }
}

//---------------------------------------------------------------------------------------------
// draw the selection when we are in wall mode
void   Rectangles::drawWallSelection(QImage* image)
{
    if (editor.isSelectSingleWall() == true)
    {
        drawWallRect(image, editor.mSelectStart, SELECTION_COLOR);
        bridge.set_wallSelScrollVis(true);
    }
    else
    {
        CRect   sel = editor.getSelectionRect();
        drawRectangleSelection(image, sel);
    }
}

//---------------------------------------------------------------------------------------------
// draw the selection when we are in object mode
void   Rectangles::drawObjectSelection(QImage* image)
{
    if (editor.isSelectSingleObject() == true)
    {
        drawObjectRect(image, editor.mSelectStart, SELECTION_COLOR);
        bridge.set_objectSelScrollVis(true);
    }
    else
    {
        CRect   sel = editor.getSelectionRect();
        drawRectangleSelection(image, sel);
    }
}

//---------------------------------------------------------------------------------------------
// draw the selection when we select a rectangular area
void   Rectangles::drawRectangleSelection(QImage* image, CRect rect)
{
    bridge.set_rectSelScrollVis(true);

    rect.tl *= TILE_SIZE;
    rect.br *= TILE_SIZE;
    rect.br += CVec2(TILE_SIZE - 1, TILE_SIZE - 1);
    graph2D.rect(image, rect, SELECTION_COLOR, false);
}

//---------------------------------------------------------------------------------------------
void Rectangles::drawSelection(QImage* image, ETabIndex tabIndex)
{
    editor.hideSelectionInfos();

    if (editor.mSelectStart.x != -1)
    {
        if (tabIndex == eTabTiles)
            drawTileSelection(image);
        else if (tabIndex == eTabWalls)
            drawWallSelection(image);
        else if (tabIndex == eTabObjects)
            drawObjectSelection(image);
    }
}

//---------------------------------------------------------------------------------------------
void Rectangles::drawPasteSelection(QImage* image)
{
    if (bridge.mTool == eQMLToolPaste &&
        bridge.mMousePos.x != -1 &&
        editor.mClipboard.isEmpty() == false)
    {
        CVec2   pos1 = bridge.mMousePos / TILE_SIZE;

        if (editor.mClipboard.mType == eTabTiles)
        {
            CVec2   pos2 = pos1 + editor.mClipboard.mSize - CVec2(1, 1);

            if (pos2.x >= map.mSize.x)
                pos2.x = map.mSize.x - 1;

            if (pos2.y >= map.mSize.y)
                pos2.y = map.mSize.y - 1;

            drawRectangleSelection(image, CRect(pos1, pos2));
        }
        else if (editor.mClipboard.mType == eTabWalls)
        {
            drawWallRect(image, bridge.mMousePos, SELECTION_COLOR);
        }
        else if (editor.mClipboard.mType == eTabObjects)
        {
            drawObjectRect(image, bridge.mMousePos, SELECTION_COLOR);
        }
    }
}
